from sangreal_wind.utils.engines import WIND_DB
from sangreal_wind.sangreal_calendar import get_all_trade_dt
from collections import Iterable


def get_fund_nav(fund_list=None, begin_dt='20010101', end_dt='20990101'):
    """[获取基金净值]

    Keyword Arguments:
        fund_list {[str or iterable or None]} -- [list of funds or fund] (default: {None})
        begin_dt {str} -- [description] (default: {'20010101'})
        end_dt {str} -- [description] (default: {'20990101'})

    Returns:
        [pd.DataFrame] -- [f_sid|trade_dt|s_close]
    """

    table = WIND_DB.CHINAMUTUALFUNDNAV
    tmp_query = WIND_DB.query(table.F_INFO_WINDCODE, table.PRICE_DATE,
                              table.F_NAV_ADJFACTOR, table.F_NAV_UNIT).filter(
                                  table.PRICE_DATE >= begin_dt,
                                  table.PRICE_DATE <= end_dt).order_by(
                                      table.PRICE_DATE, table.F_INFO_WINDCODE)
    if isinstance(fund_list, str):
        tmp_query = tmp_query.filter(table.F_INFO_WINDCODE == fund_list)
    elif isinstance(fund_list, Iterable):
        tmp_query = tmp_query.filter(table.F_INFO_WINDCODE.in_(fund_list))
    else:
        pass

    df = tmp_query.to_df()
    df.columns = ['f_sid', 'trade_dt', 'adjfactor', 'unit']
    df['s_close'] = df['adjfactor'] * df['unit']
    df.drop(['unit', 'adjfactor'], axis=1, inplace=True)
    trade_dt_list = get_all_trade_dt().trade_dt
    df = df[df.trade_dt.isin(trade_dt_list)].reset_index(drop=True)
    return df


if __name__ == '__main__':
    print(get_fund_nav('000001.OF'))
    print(get_fund_nav(['000001.OF', '000002.OF']))
    print(get_fund_nav(begin_dt='20190114'))



import pandas as pd
from sangreal_wind.utils.engines import WIND_DB
from collections import Iterable


def get_index_data_tmp(index_list=None,
                       index_type='normal',
                       begin_dt='20010101',
                       end_dt='20990101'):
    """
    index_list 指数列表
    index_type 指数类型 1. 普通指数 2. 申万指数 3.中信指数
    """
    if index_type == 'normal':
        table = getattr(WIND_DB, 'AindexEODPrices'.upper())
    elif index_type == 'zx':
        table = getattr(WIND_DB, "AindexIndustriesEODCITICS".upper())
    elif index_type == "wind":
        table = getattr(WIND_DB, "AindexWindIndustriesEOD".upper())
    elif index_type == 'sw':
        table = getattr(WIND_DB, "ASWSIndexEOD".upper())

    tmp_query = WIND_DB.query(
        table.S_INFO_WINDCODE, table.TRADE_DT, table.S_DQ_OPEN,
        table.S_DQ_HIGH, table.S_DQ_LOW, table.S_DQ_CLOSE,
        table.S_DQ_VOLUME, table.S_DQ_AMOUNT).filter(
            table.TRADE_DT >= begin_dt, table.TRADE_DT <= end_dt).order_by(
                table.TRADE_DT, table.S_INFO_WINDCODE)
    if isinstance(index_list, str):
        tmp_query = tmp_query.filter(table.S_INFO_WINDCODE == index_list)
    elif isinstance(index_list, Iterable):
        tmp_query = tmp_query.filter(table.S_INFO_WINDCODE.in_(index_list))
    df = tmp_query.to_df()
    df.columns = [
        'sid', 'trade_dt', 's_open', 's_high', 's_low', 's_close', 's_volume',
        's_amount'
    ]
    return df


def get_index_data(index_list, begin_dt='20010101', end_dt='20990101'):
    """[获取指数数据 高开低收]
    
    Arguments:
        index_list {[str or list]} -- [windcode of index_list]
    
    Keyword Arguments:
        begin_dt {[str]} -- [description] (default: {'20010101'})
        end_dt {[str]} -- [description] (default: {'20990101'})
    
    Returns:
        [pd.DataFrame] -- ['sid', 'trade_dt', 's_open', 's_high', 's_low', 's_close', 's_volume', 's_amount']
    """

    if isinstance(index_list, str):
        index_list = {index_list}
    else:
        index_list = set(index_list)
    sw_list = set((i for i in index_list if i.endswith('SI')))
    zx_list = set((i for i in index_list if i.startswith('CI')))
    wind_list = set((i for i in index_list if i.startswith('88')))
    n_list = index_list - sw_list - zx_list - wind_list
    sw_df = get_index_data_tmp(
        index_list=sw_list, index_type='sw', begin_dt=begin_dt,
        end_dt=end_dt) if len(sw_list) > 0 else pd.DataFrame()
    wind_df = get_index_data_tmp(
        index_list=wind_list,
        index_type='wind',
        begin_dt=begin_dt,
        end_dt=end_dt) if len(wind_list) > 0 else pd.DataFrame()
    zx_df = get_index_data_tmp(
        index_list=zx_list, index_type='zx', begin_dt=begin_dt,
        end_dt=end_dt) if len(zx_list) > 0 else pd.DataFrame()
    n_df = get_index_data_tmp(
        index_list=n_list,
        index_type='normal',
        begin_dt=begin_dt,
        end_dt=end_dt) if len(n_list) > 0 else pd.DataFrame()
    df = pd.concat([wind_df, sw_df, zx_df, n_df],
                   axis=0,
                   ignore_index=True,
                   sort=False)
    return df


if __name__ == '__main__':
    print(get_index_data('000001.SH', ))



from sangreal_wind.utils.engines import WIND_DB
from sangreal_wind.utils.fund_type import FUND_TYPE
from fastcache import lru_cache
from collections import Iterable

FUND_TYPE_LEVEL0 = ['股票型基金', '混合型基金', '债券型基金']


@lru_cache()
def get_fund_list():
    table0 = getattr(WIND_DB, 'ChinaMutualFundSector'.upper())
    table1 = getattr(WIND_DB, 'AShareIndustriesCode'.upper())
    df = WIND_DB.query(table0.F_INFO_WINDCODE, table0.S_INFO_SECTORENTRYDT,
                       table0.S_INFO_SECTOREXITDT,
                       table1.INDUSTRIESNAME).filter(
                           table0.S_INFO_SECTOR == table1.INDUSTRIESCODE,
                           table0.S_INFO_SECTORENTRYDT != None,
                           table0.S_INFO_SECTOREXITDT == None).to_df()
    df.columns = [c.lower() for c in df.columns]
    df = df[df['INDUSTRIESNAME'.lower()].isin(FUND_TYPE)]
    df.columns = ['sid', 'entry_dt', 'exit_dt', 'fund_type']
    return df


def get_fund_filter(fundtype='all'):
    """[选取同一类型下的基金]
    
    Keyword Arguments:
        fundtype {str} -- [基金类型] (default: {'all'})
    
    Raises:
        ValueError -- [description]
    
    Returns:
        [pd.Series] -- [Series of fund]
    """

    df = get_fund_list()
    if fundtype == 'all':
        return df.sid
    elif fundtype == '股票型':
        return df[df['fund_type'].isin((
            '普通股票型基金',
            '被动指数型基金',
            '增强指数型基金',
        ))].sid
    elif fundtype == '混合型':
        return df[df['fund_type'].isin((
            '偏股混合型基金',
            '平衡混合型基金',
            '偏债混合型基金',
            '灵活配置型基金',
        ))].sid
    elif fundtype == '债券型':
        return df[df['fund_type'].isin((
            '中长期纯债型基金',
            '短期纯债型基金',
            '混合债券型一级基金',
            '混合债券型二级基金',
            '被动指数型债券基金',
            '增强指数型债券基金',
        ))].sid
    elif isinstance(fundtype, str):
        tmp_f = fundtype.rstrip('基金') + '基金'
        return df[df['fund_type'] == tmp_f].sid
    elif isinstance(fundtype, Iterable):
        tmp_fundtype = [f.rstrip('基金') + '基金' for f in fundtype]
        return df[df['fund_type'].isin(tmp_fundtype)].sid
    else:
        raise ValueError(f'请输入正确的基金类型！ 如{FUND_TYPE_LEVEL0 + FUND_TYPE}')


if __name__ == '__main__':
    print(get_fund_filter('all').head())
    print(get_fund_filter('债券型').head())
    print(get_fund_filter('中长期纯债型基').head())
    print(get_fund_filter(['中长期纯债型基', '中长期纯债型基']).head())



import os
from itertools import chain

import pandas as pd
from sangreal_wind import WIND_DB


def china_td():
    """获取所有交易日
  
    Returns:
        list of 'trade_dt'
    """
    table = WIND_DB.ASHARECALENDAR
    df = WIND_DB.query(
        table.TRADE_DAYS).filter(table.S_INFO_EXCHMARKET == 'SSE').order_by(
            table.TRADE_DAYS).to_df()
    df.columns = ['trade_dt']
    return df.trade_dt.tolist()


def usa_td():
    pass


# 这个要作到说明文档中
dict_td = {
    'cn': china_td,
    'us': usa_td,
}


def mixin_trade_dt():
    country_list = os.environ.get('sangreal_calendar')
    if country_list is not None:
        country_list = country_list.split(',')
    else:
        country_list = [
            'cn',
        ]
    lst = list(set(chain(*(dict_td[k]() for k in country_list))))
    lst.sort()
    return pd.DataFrame(data=lst, columns=['trade_dt'])
